@@ -14,115 +14,128 @@ Principles of Database Systems, 1986.
 import unittest, os, time, itertools, copy
 from FuXi.Rete.RuleStore import SetupRuleStore, N3RuleStore, N3Builtin, LOG
 from FuXi.Rete.AlphaNode import ReteToken
+from FuXi.Rete.BetaNode import project
 from FuXi.Horn.HornRules import Clause, Ruleset, Rule, HornFromN3
 from FuXi.DLP import FUNCTIONAL_SEMANTCS, NOMINAL_SEMANTICS
 from FuXi.Horn.PositiveConditions import *
+from FuXi.Rete.Proof import *
 from FuXi.Syntax.InfixOWL import OWL_NS
 from cStringIO import StringIO
-from rdflib.Graph import Graph
+from rdflib.Graph import Graph, ReadOnlyGraphAggregate
 from rdflib import URIRef, RDF, RDFS, Namespace, Variable, Literal, URIRef
 from rdflib.sparql.Algebra import RenderSPARQLAlgebra
 from rdflib.sparql.bison import Parse
 from rdflib.util import first
-from testMagic import *
-from SidewaysInformationPassing import BuildNaturalSIP, IncomingSIPArcs
+from SidewaysInformationPassing import *
 
 EX_ULMAN = Namespace('http://doi.acm.org/10.1145/6012.15399#')
 LOG_NS   = Namespace("http://www.w3.org/2000/10/swap/log#")
 MAGIC = Namespace('http://doi.acm.org/10.1145/28659.28689#')
 
-def MagicSetTransformation(factGraph,rules,GOALS,derivedPreds=None):
+def MagicSetTransformation(factGraph,rules,GOALS,derivedPreds=None,strictCheck=False,noMagic=[]):
     """
     Takes a goal and a ruleset and returns an iterator
     over the rulest that corresponds to the magic set
     transformation:
-    
-    [[[
-    
-    ]]]
     """
     magicPredicates=set()
     if not derivedPreds:
-        derivedPreds=list(DerivedPredicateIterator(factGraph,rules))
+        _derivedPreds=DerivedPredicateIterator(factGraph,rules)
+        if not isinstance(derivedPreds,list):
+            derivedPreds=list(_derivedPreds)
+        else:
+            derivedPreds.extend(_derivedPreds)
     replacement={}
-#    for rule in rules:
-#        if isinstance(rule.formula.head,And):
-#            from FuXi.DLP import LloydToporTransformation
-#            replacement.setdefault(rule,[]).extend([Rule(rule) 
-#                            for rule in LloydToporTransformation(rule.formula)])
-#    for initial,replacements in replacement.items():
-#        rules.remove(initial)
-#        rules.extend(replacements)
     rs=AdornProgram(factGraph,rules,GOALS,derivedPreds)
+    if factGraph:
+        factGraph.adornedProgram = rs
     newRules=[]
     for rule in rs: 
         magicPositions={}
-        prevPredicates=[]
         #Generate magic rules
         for idx,pred in enumerate(iterCondition(rule.formula.body)):
             magicBody=[]
             if isinstance(pred,AdornedUniTerm):# and pred not in magicPredicates:
                 # For each rule r in Pad, and for each occurrence of an adorned 
                 # predicate p a in its body, we generate a magic rule defining magic_p a
-                if 'b' not in pred.adornment:
-                    raise                   
                 prevPreds=[item for _idx,item in enumerate(rule.formula.body)
                                             if _idx < idx]             
-                magicPred=pred.makeMagicPred()
-                prevPredicates.append(magicPred)
-                magicPositions[idx]=(magicPred,pred)
-                inArcs=[(N,x) for (N,x) in IncomingSIPArcs(rule.sip,GetOp(pred))
-                                    if not set(x).difference(pred.arg)]
-                if len(inArcs) > 1:
-                    print rule
-                    print rule.sip.serialize(format='n3')
-                    print pred, magicPred
-                    raise NotImplementedError()
-#                        labelLiterals=[]
-#                        for idx,(N,x) in enumerate(inArcs):
-#                            if not set(x).difference(pred.arg):
-#                                ruleHead=Uniterm(pred.op+'_label_'+str(idx),[i for i in x])
-#                                labelLiterals.append(ruleHead)
-#                                ruleBody=And(buildMagicBody(
-#                                    N,
-#                                    [ rule.forumula.body],
-#                                    rule.formula.head,
-#                                    derivedPreds))
-#                                magicRules.append(Rule(Clause(ruleBody,ruleHead)))
-#                        magicRules.append(Rule(Clause(And(labelLiterals),magicPred)))
-                else:
-                    for idxSip,(N,x) in enumerate(inArcs):
-                        ruleBody=And(buildMagicBody(
-                                N,
-                                prevPreds,
-                                rule.formula.head,
-                                derivedPreds))
-                        newRules.append(Rule(Clause(ruleBody,magicPred)))
-                magicPredicates.add(magicPred)
-            else:
-                prevPredicates.append(pred)
-                
+                assert 'b' in pred.adornment,"adorned predicate w/out any bound arguments!"
+                if GetOp(pred) not in noMagic:
+                    magicPred=pred.makeMagicPred()
+                    magicPositions[idx]=(magicPred,pred)
+                    inArcs=[(N,x) for (N,x) in IncomingSIPArcs(rule.sip,getOccurrenceId(pred))
+                                        if not set(x).difference(GetArgs(pred))]
+                    if len(inArcs) > 1:
+    #                    If there are several arcs entering qi, we define the magic rule defining magic_qi
+    #                    in two steps. First, for each arc Nj --> qi with label cj , we define a rule with head label_qi_j(cj ). The body
+    #                    of the rule is the same as the body of the magic rule in the case where there is a single arc
+    #                    entering qi (described above). Then the magic rule is defined as follows. The head is
+    #                    magic_q(0). The body contains label_qi_j(cj) for all j (that is, for all arcs entering qi ).
+    
+                        #We combine all incoming arcs into a single list of (body) conditions for the magic set
+                        PrettyPrintRule(rule)
+                        SIPRepresentation(rule.sip)
+                        print pred, magicPred
+                        _body=[]
+                        additionalRules=[]
+                        for idxSip,(N,x) in enumerate(inArcs):
+                            newPred=pred.clone()
+                            SetOp(newPred,URIRef('%s_label_%s'%(newPred.op,idxSip)))
+                            ruleBody=And(buildMagicBody(
+                                    N,
+                                    prevPreds,
+                                    rule.formula.head,
+                                    derivedPreds))
+                            additionalRules.append(Rule(Clause(ruleBody,newPred)))
+                            _body.extend(newPred)                        
+    #                        _body.extend(ruleBody)
+                        additionalRules.append(Rule(Clause(And(_body),magicPred)))
+                        newRules.extend(additionalRules)
+                        for i in additionalRules:
+                            print i
+                        raise NotImplementedError()
+                    else:
+                        for idxSip,(N,x) in enumerate(inArcs):
+                            ruleBody=And(buildMagicBody(
+                                    N,
+                                    prevPreds,
+                                    rule.formula.head,
+                                    derivedPreds,
+                                    noMagic))
+                            newRule=Rule(Clause(ruleBody,magicPred))
+                            newRules.append(newRule)
+                    magicPredicates.add(magicPred)
         #Modify rules
         #we modify the original rule by inserting
         #occurrences of the magic predicates corresponding
         #to the derived predicates of the body and to the head
-        headMagicPred=rule.formula.head.makeMagicPred()
+        #If there are no bound arguments in the head, we don't modify the rule
         idxIncrement=0
         newRule=copy.deepcopy(rule)
         for idx,(magicPred,origPred) in magicPositions.items():
             newRule.formula.body.formulae.insert(idx+idxIncrement,magicPred)
             idxIncrement+=1
-        newRule.formula.body.formulae.insert(0,headMagicPred)
+        if 'b' in rule.formula.head.adornment:
+            headMagicPred=rule.formula.head.makeMagicPred()
+            if isinstance(newRule.formula.body,Uniterm):
+                newRule.formula.body = And([headMagicPred,newRule.formula.body])
+            else:
+                newRule.formula.body.formulae.insert(0,headMagicPred)
         newRules.append(newRule)
+
     if not newRules:
-        print "No magic set candidates"
-        if OWL_NS.InverseFunctionalProperty in factGraph.objects(predicate=RDF.type):
+#        print "No magic set candidates"
+        if set([OWL_NS.InverseFunctionalProperty,
+                OWL_NS.FunctionalProperty]).intersection(factGraph.objects(predicate=RDF.type)):
             newRules.extend(HornFromN3(StringIO(FUNCTIONAL_SEMANTCS)))
         if (None,OWL_NS.oneOf,None) in factGraph:
             #Only include list and oneOf semantics
             #if oneOf axiom is detected in graph 
             #reduce computational complexity
             newRules.extend(HornFromN3(StringIO(NOMINAL_SEMANTICS)))
+    elif strictCheck:
+        first(DerivedPredicateIterator(factGraph,newRules,strict=True))
     for rule in newRules:
         yield rule
 
@@ -133,7 +146,6 @@ def NormalizeGoals(goals):
     elif isinstance(goals,tuple):
         yield sparqlQuery,{}
     else:
-        print goals
         query=RenderSPARQLAlgebra(Parse(goals))
         for pattern in query.patterns:
             yield pattern[:3],query.prolog.prefixBindings
@@ -155,23 +167,6 @@ class AdornedRule(Rule):
     def __eq__(self,other):
         return hash(self) == hash(other)   
 
-def GetArgs(term):
-    if isinstance(term,N3Builtin):
-        return term.argument
-    elif isinstance(term,Uniterm):
-        return term.arg
-    else:
-        raise term        
-
-def GetOp(term):
-    if isinstance(term,N3Builtin):
-        return term.uri
-    elif isinstance(term,Uniterm):
-        return term.op
-    else:
-        print term
-        raise term        
-
 def NormalizeUniterm(term):
     if isinstance(term,Uniterm):
         return term
@@ -184,27 +179,35 @@ def AdornRule(derivedPreds,clause,newHead):
     derived predicates
     """
     assert len(list(iterCondition(clause.head)))==1
-    sip=BuildNaturalSIP(clause,derivedPreds)
+    sip=BuildNaturalSIP(clause,derivedPreds,newHead)
     bodyPredReplace={}
     for literal in iterCondition(sip.sipOrder):
         args = GetArgs(literal)
         op   = GetOp(literal)
         if op in derivedPreds:
-            for N,x in IncomingSIPArcs(sip,op): 
+            for N,x in IncomingSIPArcs(sip,getOccurrenceId(literal)): 
                 if not set(x).difference(args):
                     # A binding
                     # for q is useful, however, only if it is a binding for an argument of q.
                     bodyPredReplace[literal]=AdornedUniTerm(NormalizeUniterm(literal),
                             [ i in x and 'b' or 'f' for i in args])
-    rule=AdornedRule(Clause(And([bodyPredReplace.get(p,p) 
-                                 for p in iterCondition(sip.sipOrder)]),
-                            AdornedUniTerm(clause.head,newHead.adornment)))
+#                For a predicate occurrence with no incoming
+#                arc, the adornment contains only f. For our purposes here, 
+#                we do not distinguish between a predicate with such an 
+#                adornment and an unadorned predicate
+    if (None,RDF.type,MAGIC.SipArc) not in sip:
+        #no sip arcs
+        rule=AdornedRule(Clause(clause.body,
+                                AdornedUniTerm(clause.head,
+                                              newHead.adornment)))
+    else:                                    
+        rule=AdornedRule(Clause(And([bodyPredReplace.get(p,p) 
+                                    for p in iterCondition(sip.sipOrder)]),
+                                AdornedUniTerm(clause.head,
+                                               newHead.adornment)))
     rule.sip = sip
     return rule
 
-def GetOp(term):
-    return term.op == RDF.type and term.arg[-1] or term.op
-
 def AdornProgram(factGraph,rs,goals,derivedPreds=None):
     """
     The process starts from the given query. The query determines bindings for q, and we replace
@@ -231,52 +234,58 @@ def AdornProgram(factGraph,rs,goals,derivedPreds=None):
     from FuXi.DLP import LloydToporTransformation
 #    rs=rs is None and Ruleset(n3Rules=ruleGraph.store.rules,
 #               nsMapping=ruleGraph.store.nsMgr) or rs
-    unprocessedAdornedPreds = []
+    
+    adornedPredicateCollection=set()
     for goal,nsBindings in NormalizeGoals(goals):
-        print goal,nsBindings
-        unprocessedAdornedPreds.append(AdornLiteral(goal,nsBindings))
-        
+        adornedPredicateCollection.add(AdornLiteral(goal,nsBindings))
     if not derivedPreds:
         derivedPreds=list(DerivedPredicateIterator(factGraph,rs))
-    adornedProgram=set()
-    markedPreds=[]
-    
+    def unprocessedPreds(aPredCol):
+        rt=[]
+        for p in aPredCol:
+            if not  p.marked:
+                rt.append(p)
+        return rt
     def processedAdornedPred(pred,_list):
         for p in _list:
-            if GetOp(p) == GetOp(pred) and p.adornment == pred.adornment:
+            if GetOp(p) == GetOp(pred) and p.marked:# and p.adornment == pred.adornment:
                 return True
         return False
-    
-    while unprocessedAdornedPreds:
-        term=unprocessedAdornedPreds.pop()
-        markedPreds.append(term)
-        #check if there is a rule with term as its head
-        for rule in rs:
-            for clause in LloydToporTransformation(rule.formula):
-                head=clause.head
-                _a=GetOp(head)
-                _b=GetOp(term)
-                if isinstance(head,Uniterm) and GetOp(head) == GetOp(term):
-                    #for each rule that has p in its head, we generate an adorned version for the rule
-                    # print rule, term
-    #                print "\t",rule
-                    adornedRule=AdornRule(derivedPreds,clause,term)
-                    adornedProgram.add(adornedRule)
-                    #The adorned version of a rule contains additional adorned
-                    #predicates, and these are added
-                    for pred in iterCondition(adornedRule.formula.body):
-                        if isinstance(pred,AdornedUniTerm) and not processedAdornedPred(pred,markedPreds):
-                            unprocessedAdornedPreds.append(pred)
-                            markedPreds.append(pred)
+    toDo=unprocessedPreds(adornedPredicateCollection)
+    adornedProgram=set()
+    while toDo:
+        for term in toDo:
+            #check if there is a rule with term as its head
+            for rule in rs:
+                for clause in LloydToporTransformation(rule.formula):
+                    head=isinstance(clause.head,Exists) and clause.head.formula or clause.head
+                    _a=GetOp(head)
+                    _b=GetOp(term)
+                    if isinstance(head,Uniterm) and GetOp(head) == GetOp(term):
+                        #for each rule that has p in its head, we generate an adorned version for the rule
+                        adornedRule=AdornRule(derivedPreds,clause,term)
+                        adornedProgram.add(adornedRule)
+                        #The adorned version of a rule contains additional adorned
+                        #predicates, and these are added
+                        for pred in iterCondition(adornedRule.formula.body):
+                            aPred = not isinstance(pred,AdornedUniTerm) and AdornLiteral(pred.toRDFTuple(),nsBindings) or pred 
+                            if GetOp(pred) in derivedPreds and not processedAdornedPred(aPred,adornedPredicateCollection):
+                                adornedPredicateCollection.add(aPred)
+            term.marked=True
+        toDo=unprocessedPreds(adornedPredicateCollection)
     return adornedProgram
 
 class AdornedUniTerm(Uniterm):
     def __init__(self,uterm,adornment=None):
+        self.marked = False
         self.adornment=adornment
         self.nsMgr=uterm.nsMgr
         newArgs=copy.deepcopy(uterm.arg)
         super(AdornedUniTerm, self).__init__(uterm.op,newArgs)
         self.isMagic=False
+
+    def clone(self):
+        return AdornedUniTerm(self,self.adornment)
         
     def makeMagicPred(self):
         """
@@ -311,10 +320,15 @@ class AdornedUniTerm(Uniterm):
     #            self.arg==other.arg
                 
     def getDistinguishedVariables(self):
-        for idx,term in enumerate(self.arg):
-            if self.adornment[idx]=='b' and isinstance(term,Variable):
-                yield term
-                
+        if self.op == RDF.type:
+            for idx,term in enumerate(GetArgs(self)):
+                if self.adornment[idx]=='b' and isinstance(term,Variable):
+                    yield term
+        else:
+            for idx,term in enumerate(self.arg):
+                if self.adornment[idx]=='b' and isinstance(term,Variable):
+                    yield term
+                  
     def getBindings(self,uniterm):
         rt={}
         for idx,term in enumerate(self.arg):
@@ -334,8 +348,10 @@ class AdornedUniTerm(Uniterm):
                 
     def __repr__(self):
         pred = self.normalizeTerm(self.op)
-        adornSuffix='_'+''.join(self.adornment)
-        adornSuffix = self.op == RDF.type and '_b' or adornSuffix
+        if self.op == RDF.type:
+            adornSuffix = '_'+self.adornment[0]
+        else:
+            adornSuffix='_'+''.join(self.adornment)
         if self.isMagic:
             if self.op == RDF.type:
                 return "%s(%s)"%(self.normalizeTerm(self.arg[-1]),
@@ -381,7 +397,7 @@ def AdornLiteral(rdfTuple,newNss=None):
     args=[rdfTuple[0],rdfTuple[-1]]
     newNss=newNss is None and {} or newNss
     uTerm = BuildUnitermFromTuple(rdfTuple,newNss)
-    opArgs=rdfTuple[1] == RDF.type and [args[-1]] or args
+    opArgs=rdfTuple[1] == RDF.type and [args[0]] or args
     adornment=[ isinstance(term,(Variable,BNode)) and 'f' or 'b' 
                 for idx,term in enumerate(opArgs) ]
     return AdornedUniTerm(uTerm,adornment)  
@@ -389,7 +405,7 @@ def AdornLiteral(rdfTuple,newNss=None):
 def iterCondition(condition):
     return isinstance(condition,SetOperator) and condition or iter([condition])
 
-def DerivedPredicateIterator(factsOrBasePreds,ruleset):
+def DerivedPredicateIterator(factsOrBasePreds,ruleset,strict=False):
     """
     >>> ruleStore,ruleGraph=SetupRuleStore()
     >>> g=ruleGraph.parse(StringIO(MAGIC_PROGRAM1),format='n3')
@@ -420,6 +436,9 @@ def DerivedPredicateIterator(factsOrBasePreds,ruleset):
                     ruleBodyPreds.add(op)
                 else:
                     ruleHeads.add(op)
+                if strict and not (op not in basePreds or idx > 0):
+#                    print term," is a member of a derived predicate and a base predicate!"
+                    raise SyntaxError("%s is a member of a derived predicate and a base predicate!"%term)
 #                assert op not in basePreds or idx > 0,"Malformed program!"
                 if op in basePreds:
                     uncertainPreds.add(op)
@@ -456,7 +475,7 @@ def NormalizeLPDb(ruleGraph,fact_db):
     For performance reasons, it 1s good to decompose the database into a set of
     pure base predicates (which can then be stored using a standard DBMS)
     and a set of pure derived predicates Fortunately, such a decomposition 1s 
-    always possible, because every database can be rewritten as an ‚Äö√Ñ√∂‚àö√ë‚àö‚àÇ‚Äö√†√∂‚àö√´‚Äö√†√∂‚Äö√†¬¥equivalent‚Äö√Ñ√∂‚àö√ë‚àö‚àÇ‚Äö√†√∂‚àö√´‚Äö√†√∂≈ì√Ñ
+    always possible, because every database can be rewritten ...as 
     database containing only base and derived predicates.    
     
     >>> ruleStore,ruleGraph=SetupRuleStore()
@@ -489,15 +508,37 @@ def NormalizeLPDb(ruleGraph,fact_db):
         rs.append(newRule)
     return rs
 
+#class AdornProgramTest(unittest.TestCase):
+#    def setUp(self):
+#        self.ruleStore,self.ruleGraph=SetupRuleStore(StringIO(PROGRAM2))
+#        self.ruleStore._finalize()
+#        self.ruleStrings=[
+#        'Forall ?Y ?X ( :sg_bf(?X ?Y) :- And( :sg_magic(?X) ex:flat(?X ?Y) ) )',
+#        'Forall  ( :sg_magic(?Z1) :- And( :sg_magic(?X) ex:up(?X ?Z1) ) )',
+#        'Forall ?Z4 ?Y ?X ?Z1 ?Z2 ?Z3 ( :sg_bf(?X ?Y) :- And( :sg_magic(?X) ex:up(?X ?Z1) :sg_magic(?Z1) :sg_bf(?Z1 ?Z2) ex:flat(?Z2 ?Z3) :sg_magic(?Z3) :sg_bf(?Z3 ?Z4) ex:down(?Z4 ?Y) ) )',
+#        'Forall  ( :sg_magic(?Z3) :- And( :sg_magic(?X) ex:up(?X ?Z1) :sg_bf(?Z1 ?Z2) ex:flat(?Z2 ?Z3) ) )',
+#        ]
+#
+#    def testAdorn(self):
+#        fg=Graph().parse(StringIO(PROGRAM2),format='n3')
+#        rules=Ruleset(n3Rules=self.ruleGraph.store.rules,
+#                   nsMapping=self.ruleStore.nsMgr)
+#        from pprint import pprint;pprint(self.ruleStrings)        
+#        for rule in MagicSetTransformation(fg,
+#                                           rules,
+#                                           NON_LINEAR_MS_QUERY,
+#                                           [MAGIC.sg]):
+#            self.failUnless(repr(rule) in self.ruleStrings, repr(rule))
+
 class AdornProgramTest(unittest.TestCase):
     def setUp(self):
         self.ruleStore,self.ruleGraph=SetupRuleStore(StringIO(PROGRAM2))
         self.ruleStore._finalize()
         self.ruleStrings=[
-        'Forall ?Y ?X ( _5:sg_bf(?X ?Y) :- And( _5:sg_magic(?X) ex:flat(?X ?Y) ) )',
-        'Forall  ( _5:sg_magic(?Z1) :- And( _5:sg_magic(?X) ex:up(?X ?Z1) ) )',
-        'Forall ?Z4 ?Y ?X ?Z1 ?Z2 ?Z3 ( _5:sg_bf(?X ?Y) :- And( _5:sg_magic(?X) ex:up(?X ?Z1) _5:sg_magic(?Z1) _5:sg_bf(?Z1 ?Z2) ex:flat(?Z2 ?Z3) _5:sg_magic(?Z3) _5:sg_bf(?Z3 ?Z4) ex:down(?Z4 ?Y) ) )',
-        'Forall  ( _5:sg_magic(?Z3) :- And( _5:sg_magic(?X) ex:up(?X ?Z1) _5:sg_bf(?Z1 ?Z2) ex:flat(?Z2 ?Z3) ) )',
+        'Forall ?Y ?X ( :sg_bf(?X ?Y) :- And( :sg_magic(?X) ex:flat(?X ?Y) ) )',
+        'Forall  ( :sg_magic(?Z1) :- And( :sg_magic(?X) ex:up(?X ?Z1) ) )',
+        'Forall ?Z4 ?Y ?X ?Z1 ?Z2 ?Z3 ( :sg_bf(?X ?Y) :- And( :sg_magic(?X) ex:up(?X ?Z1) :sg_magic(?Z1) :sg_bf(?Z1 ?Z2) ex:flat(?Z2 ?Z3) :sg_magic(?Z3) :sg_bf(?Z3 ?Z4) ex:down(?Z4 ?Y) ) )',
+        'Forall  ( :sg_magic(?Z3) :- And( :sg_magic(?X) ex:up(?X ?Z1) :sg_bf(?Z1 ?Z2) ex:flat(?Z2 ?Z3) ) )',
         ]
 
     def testAdorn(self):
@@ -509,20 +550,42 @@ class AdornProgramTest(unittest.TestCase):
                                            rules,
                                            NON_LINEAR_MS_QUERY,
                                            [MAGIC.sg]):
-            self.failUnless(repr(rule) in self.ruleStrings, repr(rule.formula))
+            self.failUnless(repr(rule) in self.ruleStrings, repr(rule))
         
-def buildMagicBody(N,prevPredicates,adornedHead,derivedPreds):
-    body=[adornedHead.makeMagicPred()]
+def buildMagicBody(N,prevPredicates,adornedHead,derivedPreds,noMagic=[]):
+    unboundHead='b' in adornedHead.adornment
+    if unboundHead:
+        body=[adornedHead.makeMagicPred()]
+    else:
+        #If there are no bound argument positions to pass magic values with,
+        #we propagate values in the full relation
+        body=[]
     for prevAPred in prevPredicates:
         op = GetOp(prevAPred)
-        if op in N:
+        if op in N or isinstance(op,Variable):
             #If qj, j<i, is in N, we add qj to the body of the magic rule
+            #Note, if the atom has a variable for the predicate, treat it as a base
+            #predicate occurrence
             body.append(prevAPred)
-        if op in derivedPreds and prevAPred.adornment.count('b')>0:
+        if op in derivedPreds and isinstance(prevAPred,AdornedUniTerm) and prevAPred.adornment.count('b')>0:
             #If qj is a derived predicate and its adornment contains at least 
             #one b, we also add the corresponding magic predicate to the body
-            body.append(prevAPred.makeMagicPred())
+            if op in noMagic:
+                body.append(prevAPred)
+            else:
+                body.append(prevAPred.makeMagicPred())
     return body
+
+def PrettyPrintRule(rule):
+    if isinstance(rule.formula.body,And):
+        print rule.formula.head
+        print "    :- %s"%rule.formula.body.formulae[0]
+        for idx,literal in enumerate(rule.formula.body.formulae[1:]):
+            print "       %s%s"%(literal,
+                                 literal == rule.formula.body.formulae[-1] and '' or ', ')
+    else:
+        print rule.formula
+    
             
 def test():
     unittest.main()    
