@@ -11,14 +11,14 @@ The network :
     - stores inferred triples in provided triple source (an RDFLib graph) or a temporary IOMemory Graph by default
 
 """
-from itertools import izip,ifilter
+from itertools import izip,ifilter,chain
 import time,sys
-from sets import Set
 from pprint import pprint
 from cStringIO import StringIO
 from Util import xcombine
 from BetaNode import BetaNode, LEFT_MEMORY, RIGHT_MEMORY, PartialInstanciation
 from AlphaNode import AlphaNode, ReteToken, SUBJECT, PREDICATE, OBJECT, BuiltInAlphaNode
+from BuiltinPredicates import FILTERS
 from FuXi.Horn import ComplementExpansion
 from FuXi.Syntax.InfixOWL import *
 from FuXi.Horn.PositiveConditions import Uniterm, SetOperator, Exists, Or
@@ -135,7 +135,7 @@ def _mulPatternWithSubstitutions(tokens,consequent,termNode):
         #     return
         # else:
         for term in consequent:
-            if isinstance(term,Variable) and term in binding:
+            if isinstance(term,(Variable,BNode)) and term in binding:
                 #try:                
                 tripleVals.append(binding[term])
                 #except:
@@ -157,7 +157,7 @@ class ReteNetwork:
                  graphVizOutFile=None,
                  dontFinalize=False,
                  goal=None):
-        from BuiltinPredicates import FILTERS
+        self.leanCheck = {}
         self.goal = goal        
         self.nsMap = nsMap
         self.name = name and name or BNode()
@@ -174,7 +174,7 @@ class ReteNetwork:
             self.inferredFacts.namespace_manager = namespace_manager    
         else:            
             self.inferredFacts = inferredTarget
-        self.workingMemory = initialWorkingMemory and initialWorkingMemory or Set()
+        self.workingMemory = initialWorkingMemory and initialWorkingMemory or set()
         self.proofTracers = {}
         self.terminalNodes  = set()
         self.instanciations = {}        
@@ -184,7 +184,7 @@ class ReteNetwork:
         self.dischargedBindings = {}
         if not dontFinalize:
             self.ruleStore._finalize()
-        #self.ruleStore.optimizeRules()
+        self.filteredFacts = Graph()
         
         #'Universal truths' for a rule set are rules where the LHS is empty.  
         # Rather than automatically adding them to the working set, alpha nodes are 'notified'
@@ -192,11 +192,14 @@ class ReteNetwork:
         self.universalTruths = []
         from FuXi.Horn.HornRules import Ruleset
         self.rules=set()
+        self.negRules = set()
         for rule in Ruleset(n3Rules=self.ruleStore.rules,nsMapping=self.nsMap):
-            self.buildNetwork(iter(rule.formula.body),
-                              iter(rule.formula.head),
-                              rule.formula)
-            self.rules.add(rule)
+            import warnings
+            warnings.warn(
+          "Rules in a network should be built *after* construction via "+
+          " self.buildNetworkClause(HornFromN3(n3graph)) for instance",
+                          DeprecationWarning,2)            
+            self.buildNetworkFromClause(rule)
         self.alphaNodes = [node for node in self.nodes.values() if isinstance(node,AlphaNode)]
         self.alphaBuiltInNodes = [node for node in self.nodes.values() if isinstance(node,BuiltInAlphaNode)]
         self._setupDefaultRules()
@@ -212,33 +215,135 @@ class ReteNetwork:
     def getNsBindings(self,nsMgr):
         for prefix,Uri in nsMgr.namespaces():
             self.nsMap[prefix]=Uri
-                        
-    def buildNetworkFromClause(self,horn_clause):
+
+    def buildFilterNetworkFromClause(self,rule):
         lhs = BNode()
         rhs = BNode()
-        for term in horn_clause.body:
-            self.ruleStore.formulae.setdefault(lhs,Formula(lhs)).append(term.toRDFTuple())
-        for term in horn_clause.head:
+        builtins=[]
+        for term in rule.formula.body:
+            if isinstance(term, N3Builtin):
+                #We want to move builtins to the 'end' of the body
+                #so they only apply to the terminal nodes of 
+                #the corresponding network
+                builtins.append(term)
+            else:
+                self.ruleStore.formulae.setdefault(lhs,Formula(lhs)).append(term.toRDFTuple())
+        for builtin in builtins:
+            self.ruleStore.formulae.setdefault(lhs,Formula(lhs)).append(builtin.toRDFTuple())
+        nonEmptyHead=False
+        for term in rule.formula.head:
+            nonEmptyHead=True
             assert not hasattr(term,'next')
             assert isinstance(term,Uniterm)
             self.ruleStore.formulae.setdefault(rhs,Formula(rhs)).append(term.toRDFTuple())
+        assert nonEmptyHead,"Filters must conclude something!"
         self.ruleStore.rules.append((self.ruleStore.formulae[lhs],self.ruleStore.formulae[rhs]))
         self.buildNetwork(iter(self.ruleStore.formulae[lhs]),
                              iter(self.ruleStore.formulae[rhs]),
-                             horn_clause)
+                             rule,
+                             filter=True)
         self.alphaNodes = [node for node in self.nodes.values() if isinstance(node,AlphaNode)]
+        self.rules.add(rule)
+                        
+    def buildNetworkFromClause(self,rule):
+        lhs = BNode()
+        rhs = BNode()
+        builtins=[]
+        for term in rule.formula.body:
+            if isinstance(term, N3Builtin):
+                #We want to move builtins to the 'end' of the body
+                #so they only apply to the terminal nodes of 
+                #the corresponding network
+                builtins.append(term)
+            else:
+                self.ruleStore.formulae.setdefault(lhs,Formula(lhs)).append(term.toRDFTuple())
+        for builtin in builtins:
+            self.ruleStore.formulae.setdefault(lhs,Formula(lhs)).append(builtin.toRDFTuple())
+        nonEmptyHead=False
+        for term in rule.formula.head:
+            nonEmptyHead=True
+            assert not hasattr(term,'next')
+            assert isinstance(term,Uniterm)
+            self.ruleStore.formulae.setdefault(rhs,Formula(rhs)).append(term.toRDFTuple())
+        if not nonEmptyHead:
+            import warnings
+            warnings.warn(
+          "Integrity constraints (rules with empty heads) are not supported!: %s"%rule,
+                          SyntaxWarning,2)            
+            return
+        self.ruleStore.rules.append((self.ruleStore.formulae[lhs],self.ruleStore.formulae[rhs]))
+        self.buildNetwork(iter(self.ruleStore.formulae[lhs]),
+                             iter(self.ruleStore.formulae[rhs]),
+                             rule)
+        self.alphaNodes = [node for node in self.nodes.values() if isinstance(node,AlphaNode)]
+        self.rules.add(rule)
+        
+    def calculateStratifiedModel(self,database):
+        """
+        Stratified Negation Semantics for DLP using SPARQL to handle the negation
+        """
+        if not self.negRules:
+            return
+        from FuXi.DLP.Negation import StratifiedSPARQL
+        from FuXi.Rete.Magic import PrettyPrintRule
+        import copy
+        noNegFacts = 0
+        for i in self.negRules:
+            #Evaluate the Graph pattern, and instanciate the head of the rule with 
+            #the solutions returned
+            sel,compiler=StratifiedSPARQL(i,dict([(v,k) 
+                                              for k,v in self.nsMap.items()]))
+            query=compiler.compile(sel)
+            i.stratifiedQuery=query
+            vars = sel.projection
+            unionClosureG = self.closureGraph(database)
+            for rt in unionClosureG.query(query):
+                solutions={}
+                if isinstance(rt,tuple):
+                    solutions.update(dict([(vars[idx],i) for idx,i in enumerate(rt)]))
+                else:
+                    solutions[vars[0]]=rt
+                i.solutions=solutions
+                head=copy.deepcopy(i.formula.head)
+                head.ground(solutions)
+                fact=head.toRDFTuple()
+                self.inferredFacts.add(fact)
+                self.feedFactsToAdd(generateTokenSet([fact]))
+                noNegFacts += 1
+        #Now we need to clear assertions that cross the individual, concept, relation divide
+        toRemove=[]
+        for s,p,o in self.inferredFacts.triples((None,RDF.type,None)):
+            if s in unionClosureG.predicates() or\
+               s in [_s for _s,_p,_o in 
+                        unionClosureG.triples_choices(
+                                            (None,
+                                             RDF.type,
+                                             [OWL_NS.Class,
+                                              OWL_NS.Restriction]))]:
+                self.inferredFacts.remove((s,p,o))
+        return noNegFacts
                         
     def setupDescriptionLogicProgramming(self,
                                          owlN3Graph,
                                          expanded=[],
                                          addPDSemantics=True,
                                          classifyTBox=False,
-                                         constructNetwork=True):
-        rules=[rule 
+                                         constructNetwork=True,
+                                         derivedPreds=[],
+                                         ignoreNegativeStratus=False):
+        rt=[rule 
                     for rule in MapDLPtoNetwork(self,
                                                 owlN3Graph,
                                                 complementExpansions=expanded,
-                                                constructNetwork=constructNetwork)]
+                                                constructNetwork=constructNetwork,
+                                                derivedPreds=derivedPreds,
+                                                ignoreNegativeStratus=ignoreNegativeStratus)]
+        if ignoreNegativeStratus:
+            rules,negRules=rt
+            rules = [r for r in rules]
+            self.negRules = set(negRules)
+        else:
+            rules=rt
         if constructNetwork:
             self.rules.update(rules)
         noRules=len(rules)
@@ -255,18 +360,17 @@ class ReteNetwork:
 #        print "##### DLP rules fired against OWL/RDF TBOX",self
         return rules
     
-    def reportConflictSet(self,closureSummary=False):
+    def reportConflictSet(self,closureSummary=False,stream=sys.stdout):
         tNodeOrder = [tNode 
                         for tNode in self.terminalNodes 
                             if self.instanciations.get(tNode,0)]
         tNodeOrder.sort(key=lambda x:self.instanciations[x],reverse=True)
         for termNode in tNodeOrder:
-            print >>sys.stdout,termNode
-            #print "\t %s => %s"%(lhsF,rhsF)
-            print >>sys.stdout,"\t", termNode.clause
-            print >>sys.stdout,"\t\t%s instanciations"%self.instanciations[termNode]
+            print >>stream,termNode
+            print >>stream,"\t", termNode.clause
+            print >>stream,"\t\t%s instanciations"%self.instanciations[termNode]
         if closureSummary:        
-            print self.inferredFacts.serialize(format='turtle')
+            print >>stream ,self.inferredFacts.serialize(format='turtle')
                 
     def parseN3Logic(self,src):
         store=N3RuleStore(additionalBuiltins=self.ruleStore.filters)
@@ -278,7 +382,7 @@ class ReteNetwork:
                             nsMapping=self.nsMap):
             self.buildNetwork(iter(rule.formula.body),
                               iter(rule.formula.head),
-                              rule.formula)
+                              rule)
             self.rules.add(rule)
         self.alphaNodes = [node for node in self.nodes.values() if isinstance(node,AlphaNode)]
         self.alphaBuiltInNodes = [node for node in self.nodes.values() if isinstance(node,BuiltInAlphaNode)]        
@@ -294,8 +398,12 @@ class ReteNetwork:
         
     def closureGraph(self,sourceGraph,readOnly=True,store=None):
         if readOnly:
+            if store is None and not sourceGraph:
+                store = Graph().store
+            store = store is None and sourceGraph.store or store
             roGraph=ReadOnlyGraphAggregate([sourceGraph,self.inferredFacts],
                                            store=store)
+            roGraph.namespace_manager = NamespaceManager(roGraph)
             for srcGraph in [sourceGraph,self.inferredFacts]:
                 for prefix,uri in srcGraph.namespaces():
                     roGraph.namespace_manager.bind(prefix,uri)
@@ -317,12 +425,14 @@ class ReteNetwork:
     def clear(self):
         self.nodes = {}
         self.alphaPatternHash = {}
-        self.ruleSet = set()
+        self.rules = set()
         for alphaPattern in xcombine(('1','0'),('1','0'),('1','0')):
             self.alphaPatternHash[tuple(alphaPattern)] = {}
         self.proofTracers = {}
         self.terminalNodes  = set()
         self.justifications = {}
+        self._resetinstanciationStats()
+        self.workingMemory = set()
         self.dischargedBindings = {}
         
     def reset(self,newinferredFacts=None):
@@ -331,9 +441,10 @@ class ReteNetwork:
             if isinstance(node,BetaNode):
                 node.memories[LEFT_MEMORY].reset()
                 node.memories[RIGHT_MEMORY].reset()
-        self.inferredFacts = newinferredFacts and newinferredFacts or Graph('IOMemory')
-        self.workingMemory = Set()
-        self.rules = Set()
+        self.justifications = {}
+        self.proofTracers = {}
+        self.inferredFacts = newinferredFacts and newinferredFacts or Graph()
+        self.workingMemory = set()
         self._resetinstanciationStats()        
                                 
     def fireConsequent(self,tokens,termNode,debug=False):
@@ -373,7 +484,10 @@ class ReteNetwork:
         #replace existentials in the head with new BNodes!
         BNodeReplacement = {}
         if isinstance(clause.head,Exists):
-            BNodeReplacement = dict([(bN,BNode()) for bN in clause.head.declare])
+            BNodeReplacement = dict([(bN,BNode()) 
+                                        for bN in clause.head.declare 
+                                            if not isinstance(clause.body,Exists) or 
+                                               bN not in clause.body.declare])
         for rhsTriple in termNode.consequent:
             if BNodeReplacement:
                 rhsTriple = tuple([BNodeReplacement.get(term,term) for term in rhsTriple])
@@ -384,18 +498,20 @@ class ReteNetwork:
                 if [term for term in inferredTriple if isinstance(term,Variable)]:
                     #Unfullfilled bindings
                     continue
-                #print inferredTriple
                 inferredToken=ReteToken(inferredTriple)
                 self.proofTracers.setdefault(inferredTriple,[]).append(binding)
                 self.justifications.setdefault(inferredTriple,set()).add(termNode)
-                currIdx = self.instanciations.get(termNode,0)
-                currIdx+=1
-                self.instanciations[termNode] = currIdx
+                if termNode.filter and inferredTriple not in self.filteredFacts:
+                    self.filteredFacts.add(inferredTriple)
                 if inferredTriple not in self.inferredFacts and inferredToken not in self.workingMemory:                    
                     if debug:
                         print "Inferred triple: ", inferredTriple, " from ",termNode 
                     self.inferredFacts.add(inferredTriple)
                     self.addWME(inferredToken)
+                    currIdx = self.instanciations.get(termNode,0)
+                    currIdx+=1
+                    self.instanciations[termNode] = currIdx
+                    
 #                    if self.goal is not None and self.goal in self.inferredFacts:
 #                        return                    
                 elif debug:
@@ -454,8 +570,11 @@ class ReteNetwork:
         if isinstance(currentPattern,N3Builtin):
             node = BuiltInAlphaNode(currentPattern)
         else:
-            node = AlphaNode(currentPattern)
+            node = AlphaNode(currentPattern,self.ruleStore.filters)
         self.alphaPatternHash[node.alphaNetworkHash()].setdefault(node.alphaNetworkHash(groundTermHash=True),[]).append(node)
+        if not isinstance(node,BuiltInAlphaNode) and node.builtin:
+            s,p,o = currentPattern
+            node = BuiltInAlphaNode(N3Builtin(p,self.ruleStore.filters[p](s,o),s,o))
         return node
     
     def _resetinstanciationStats(self):
@@ -468,7 +587,7 @@ class ReteNetwork:
             assert collision is None,"%s collides with %s"%(tNode,checkedClauses[tNode.clause])
             checkedClauses.setdefault(tNode.clause,[]).append(tNode) 
     
-    def buildNetwork(self,lhsIterator,rhsIterator,clause):
+    def buildNetwork(self,lhsIterator,rhsIterator,rule,filter=False):
         """
         Takes an iterator of triples in the LHS of an N3 rule and an iterator of the RHS and extends
         the Rete network, building / reusing Alpha 
@@ -504,28 +623,35 @@ class ReteNetwork:
                     else:
                         paddedLHSPattern = HashablePatternList([None])+attachedPatterns[0]                    
                         terminalNode = self.nodes.get(paddedLHSPattern,BetaNode(None,node,aPassThru=True))
-                        if True:#clause not in self.ruleSet:
-                            self.nodes[paddedLHSPattern] = terminalNode    
-                            node.connectToBetaNode(terminalNode,RIGHT_MEMORY)                            
-                    if True:#clause not in self.ruleSet:
-                        #self.ruleSet.add(clause)                        
-                        terminalNode.rule = (LHS,consequents)
-                        terminalNode.consequent.update(consequents)
-                        terminalNode.network    = self
-                        terminalNode.clause = clause
-                        self.terminalNodes.add(terminalNode)                    
+                        self.nodes[paddedLHSPattern] = terminalNode    
+                        node.connectToBetaNode(terminalNode,RIGHT_MEMORY)                            
+                    terminalNode.rule = (LHS,consequents)
+                    terminalNode.consequent.update(consequents)
+                    terminalNode.network    = self
+                    terminalNode.clause = rule.formula
+                    terminalNode.horn_rule = rule
+                    terminalNode.filter = filter
+                    self.terminalNodes.add(terminalNode)                    
                 else:              
-                    for aP in attachedPatterns:
-                        assert isinstance(aP,HashablePatternList),repr(aP)                    
-                    terminalNode = self.attachBetaNodes(iter(attachedPatterns))
-                    if True:#clause not in self.ruleSet:
-                        #self.ruleSet.add(clause)
-                        terminalNode.rule = (LHS,consequents)
-                        terminalNode.consequent.update(consequents)
-                        terminalNode.network    = self
-                        terminalNode.clause = clause
-                        self.terminalNodes.add(terminalNode)
-                        self._resetinstanciationStats()                        
+                    moveToEnd = []
+                    endIdx = len(attachedPatterns) - 1
+                    finalPatternList = []
+                    for idx,pattern in enumerate(attachedPatterns):
+                        assert isinstance(pattern,HashablePatternList),repr(pattern)                    
+                        currNode = self.nodes[pattern]
+                        if isinstance(currNode,BuiltInAlphaNode) and idx < endIdx:
+                            moveToEnd.append(pattern)
+                        else:
+                            finalPatternList.append(pattern)
+                    terminalNode = self.attachBetaNodes(chain(finalPatternList,moveToEnd))
+                    terminalNode.rule = (LHS,consequents)
+                    terminalNode.consequent.update(consequents)
+                    terminalNode.network    = self
+                    terminalNode.clause = rule.formula
+                    terminalNode.horn_rule = rule
+                    terminalNode.filter = filter
+                    self.terminalNodes.add(terminalNode)
+                    self._resetinstanciationStats()                        
                 #self.checkDuplicateRules()
                 return
             if HashablePatternList([currentPattern]) in self.nodes:
