@@ -104,7 +104,7 @@ Restrictions can also be created using Manchester OWL syntax in 'colloquial' Pyt
 #>>> print g.serialize(format='pretty-xml')
 
 """
-import os
+import os, itertools
 from pprint import pprint
 from rdflib import Namespace
 from rdflib import plugin,RDF,RDFS,URIRef,BNode,Literal,Variable
@@ -172,7 +172,7 @@ def classOrIdentifier(thing):
     if isinstance(thing,(Property,Class)):
         return thing.identifier
     else:
-        assert isinstance(thing,(URIRef,BNode)),"Expecting a Class, Property, URIRef, or BNode.."
+        assert isinstance(thing,(URIRef,BNode)),"Expecting a Class, Property, URIRef, or BNode.. not a %s"%thing
         return thing
 
 def propertyOrIdentifier(thing):
@@ -230,6 +230,8 @@ def manchesterSyntax(thing,store,boolean=None,transientList=False):
         propString = u':'.join([prefix,localName])
         for onlyClass in store.objects(subject=thing, predicate=OWL_NS.allValuesFrom):
             return '( %s only %s )'%(propString,manchesterSyntax(onlyClass,store))
+        for val in store.objects(subject=thing, predicate=OWL_NS.hasValue):
+            return '( %s value %s )'%(propString,manchesterSyntax(val,store))        
         for someClass in store.objects(subject=thing, predicate=OWL_NS.someValuesFrom):    
             return '( %s some %s )'%(propString,manchesterSyntax(someClass,store))
         cardLookup = {OWL_NS.maxCardinality:'max',OWL_NS.minCardinality:'min',OWL_NS.cardinality:'equals'}
@@ -264,11 +266,28 @@ def GetIdentifiedClasses(graph):
         if isinstance(c,URIRef):
             yield Class(c)
 
+def termDeletionDecorator(prop):
+    def someFunc(func):
+        func.property = prop
+        return func
+    return someFunc
+
+class TermDeletionHelper:
+    def __init__(self, prop):
+        self.prop = prop
+    def __call__(self, f):
+        def _remover(inst):
+            inst.graph.remove((inst.identifier,self.prop,None))
+        return _remover
+
 class Individual(object):
     """
     A typed individual
     """
     factoryGraph = Graph()
+    def serialize(self,graph):
+        for fact in self.factoryGraph.triples((self.identifier,None,None)):
+            graph.add(fact)
     def __init__(self, identifier=None,graph=None):
         self.__identifier = identifier is not None and identifier or BNode()
         if graph is None:
@@ -283,6 +302,21 @@ class Individual(object):
             except:
                 pass
     
+    def clearInDegree(self):
+        self.graph.remove((None,None,self.identifier))    
+
+    def clearOutDegree(self):
+        self.graph.remove((self.identifier,None,None))    
+
+    def delete(self):
+        self.clearInDegree()
+        self.clearOutDegree()
+        
+    def replace(self,other):
+        for s,p,o in self.graph.triples((None,None,self.identifier)):
+            self.graph.add((s,p,classOrIdentifier(other)))
+        self.delete()
+    
     def _get_type(self):
         for _t in self.graph.objects(subject=self.identifier,predicate=RDF.type):
             yield _t
@@ -295,7 +329,22 @@ class Individual(object):
             for c in kind:
                 assert isinstance(c,(Individual,Identifier))
                 self.graph.add((self.identifier,RDF.type,classOrIdentifier(c)))
-    type = property(_get_type, _set_type)
+                
+    @TermDeletionHelper(RDF.type)            
+    def _delete_type(self):
+        """
+        >>> g = Graph()
+        >>> b=Individual(OWL_NS.Restriction,g)
+        >>> b.type = RDF.Resource
+        >>> len(list(b.type))
+        1
+        >>> del b.type
+        >>> len(list(b.type))
+        0
+        """
+        pass
+                
+    type = property(_get_type, _set_type, _delete_type)
 
     def _get_identifier(self):
         return self.__identifier
@@ -311,13 +360,40 @@ class Individual(object):
             self.__identifier = i
             self.graph.addN([(i,p1,o1,self.graph) for p1,o1 in oldStmtsOut])
             self.graph.addN([(s1,p1,i,self.graph) for s1,p1 in oldStmtsIn])
+        if not isinstance(i,BNode):
+            try:
+                prefix,uri,localName = self.graph.compute_qname(i) 
+                self.qname = u':'.join([prefix,localName])
+            except:
+                pass
+            
     identifier = property(_get_identifier, _set_identifier)
+    
+    def _get_sameAs(self):
+        for _t in self.graph.objects(subject=self.identifier,predicate=OWL_NS.sameAs):
+            yield _t
+    def _set_sameAs(self, term):
+        if not kind:
+            return  
+        if isinstance(term,(Individual,Identifier)):
+            self.graph.add((self.identifier,OWL_NS.sameAs,classOrIdentifier(term)))
+        else:
+            for c in term:
+                assert isinstance(c,(Individual,Identifier))
+                self.graph.add((self.identifier,OWL_NS.sameAs,classOrIdentifier(c)))
+                
+    @TermDeletionHelper(OWL_NS.sameAs)            
+    def _delete_sameAs(self):
+        pass
+                
+    sameAs = property(_get_sameAs, _set_sameAs, _delete_sameAs)
+    
 
 class AnnotatibleTerms(Individual):
     """
     Terms in an OWL ontology with rdfs:label and rdfs:comment
     """
-    def __init__(self,identifier,graph):
+    def __init__(self,identifier,graph=None):
         super(AnnotatibleTerms, self).__init__(identifier,graph)
     def _get_comment(self):
         for comment in self.graph.objects(subject=self.identifier,predicate=RDFS.comment):
@@ -330,7 +406,12 @@ class AnnotatibleTerms(Individual):
         else:
             for c in comment:
                 self.graph.add((self.identifier,RDFS.comment,c))
-    comment = property(_get_comment, _set_comment)
+                
+    @TermDeletionHelper(RDFS.comment)
+    def _del_comment(self):
+        pass
+    
+    comment = property(_get_comment, _set_comment, _del_comment)
 
     def _get_seeAlso(self):
         for sA in self.graph.objects(subject=self.identifier,predicate=RDFS.seeAlso):
@@ -340,7 +421,11 @@ class AnnotatibleTerms(Individual):
             return        
         for s in seeAlsos:
             self.graph.add((self.identifier,RDFS.seeAlso,s))
-    seeAlso = property(_get_seeAlso, _set_seeAlso)
+            
+    @TermDeletionHelper(RDFS.seeAlso)
+    def _del_seeAlso(self):
+        pass
+    seeAlso = property(_get_seeAlso, _set_seeAlso, _del_seeAlso)
 
     def _get_label(self):
         for label in self.graph.objects(subject=self.identifier,predicate=RDFS.label):
@@ -353,14 +438,29 @@ class AnnotatibleTerms(Individual):
         else:
             for l in label:
                 self.graph.add((self.identifier,RDFS.label,l))
-    label = property(_get_label, _set_label)
+                
+    @TermDeletionHelper(RDFS.label)
+    def _delete_label(self):
+        """
+        >>> g=Graph()
+        >>> b=Individual(OWL_NS.Restriction,g)
+        >>> b.label = Literal('boo')
+        >>> len(list(b.label))
+        1
+        >>> del b.label
+        >>> len(list(b.label))
+        0
+        """
+        pass
+        
+    label = property(_get_label, _set_label, _delete_label)
 
 class Ontology(AnnotatibleTerms):
     """ The owl ontology metadata"""
     def __init__(self, identifier=None,imports=None,comment=None,graph=None):
         super(Ontology, self).__init__(identifier,graph)
         self.imports = imports and imports or []
-        self.comment = imports and imports or []
+        self.comment = comment and comment or []
         if (self.identifier,RDF.type,OWL_NS.Ontology) not in self.graph:
             self.graph.add((self.identifier,RDF.type,OWL_NS.Ontology))
 
@@ -375,7 +475,12 @@ class Ontology(AnnotatibleTerms):
             return        
         for o in other:
             self.graph.add((self.identifier,OWL_NS['imports'],o))
-    imports = property(_get_imports, _set_imports)
+            
+    @TermDeletionHelper(OWL_NS['imports'])
+    def _del_imports(self):            
+        pass
+    
+    imports = property(_get_imports, _set_imports, _del_imports)
 
 def AllClasses(graph):
     prevClasses=set()
@@ -449,10 +554,10 @@ class Class(AnnotatibleTerms):
     
     See: http://owl-workshop.man.ac.uk/acceptedLong/submission_9.pdf:
     
-    ‘Class:’ classID {Annotation
-                  ( (‘SubClassOf:’ ClassExpression)
-                  | (‘EquivalentTo’ ClassExpression)
-                  | (’DisjointWith’ ClassExpression)) }
+    ‚ÄòClass:‚Äô classID {Annotation
+                  ( (‚ÄòSubClassOf:‚Äô ClassExpression)
+                  | (‚ÄòEquivalentTo‚Äô ClassExpression)
+                  | (‚ÄôDisjointWith‚Äô ClassExpression)) }
     
     Appropriate excerpts from OWL Reference:
     
@@ -468,11 +573,27 @@ class Class(AnnotatibleTerms):
       description."
       
     """
+    def _serialize(self,graph):
+        for cl in self.subClassOf:
+            CastClass(cl,self.graph).serialize(graph)
+        for cl in self.equivalentClass:
+            CastClass(cl,self.graph).serialize(graph)
+        for cl in self.disjointWith:
+            CastClass(cl,self.graph).serialize(graph)
+        if self.complementOf:
+            CastClass(self.complementOf,self.graph).serialize(graph)
+        
+    def serialize(self,graph):
+        for fact in self.graph.triples((self.identifier,None,None)):
+            graph.add(fact)
+        self._serialize(graph)
+    
     def __init__(self, identifier=None,subClassOf=None,equivalentClass=None,
                        disjointWith=None,complementOf=None,graph=None,
                        skipOWLClassMembership = False,comment=None):
         super(Class, self).__init__(identifier,graph)
-        if not skipOWLClassMembership and (self.identifier,RDF.type,OWL_NS.Class) not in self.graph:
+        if not skipOWLClassMembership and (self.identifier,RDF.type,OWL_NS.Class) not in self.graph and \
+           (self.identifier,RDF.type,OWL_NS.Restriction) not in self.graph:
             self.graph.add((self.identifier,RDF.type,OWL_NS.Class))
         
         self.subClassOf      = subClassOf and subClassOf or [] 
@@ -482,17 +603,34 @@ class Class(AnnotatibleTerms):
             self.complementOf    = complementOf
         self.comment = comment and comment or []
         
-    def _get_extent(self):
-        for member in self.graph.subjects(predicate=RDF.type,
+    def _get_extent(self,graph=None):
+        for member in (graph is None and self.graph or graph).subjects(predicate=RDF.type,
                                           object=self.identifier):
             yield member
     def _set_extent(self,other):
         if not other:
             return
         for m in other:
-            self.graph.add((m,RDF.type,self.identifier))
+            self.graph.add((classOrIdentifier(m),RDF.type,self.identifier))
             
-    extent = property(_get_extent, _set_extent)            
+    @TermDeletionHelper(RDF.type)
+    def _del_type(self):            
+        pass            
+            
+    extent = property(_get_extent, _set_extent, _del_type)            
+
+    def _get_annotation(self,term=RDFS.label):
+        for annotation in self.graph.objects(subject=self,predicate=term):
+            yield annotation
+            
+    annotation = property(_get_annotation,lambda x:x)            
+
+    def _get_extentQuery(self):
+        return (Variable('CLASS'),RDF.type,self.identifier)
+
+    def _set_extentQuery(self,other): pass
+            
+    extentQuery = property(_get_extentQuery, _set_extentQuery)            
     
     def __hash__(self):
         """
@@ -534,18 +672,45 @@ class Class(AnnotatibleTerms):
         """
         Construct an anonymous class description consisting of the intersection of this class and '
         other' and return it
+        
+        >>> exNs = Namespace('http://example.com/')        
+        >>> namespace_manager = NamespaceManager(Graph())
+        >>> namespace_manager.bind('ex', exNs, override=False)
+        >>> namespace_manager.bind('owl', OWL_NS, override=False)
+        >>> g = Graph()    
+        >>> g.namespace_manager = namespace_manager
+        
+        Chaining 3 intersections
+                
+        >>> female      = Class(exNs.Female,graph=g)
+        >>> male        = Class(exNs.Human,graph=g)
+        >>> youngPerson = Class(exNs.YoungPerson,graph=g)
+        >>> youngWoman = female & male & youngPerson
+        >>> youngWoman
+        ex:YoungPerson that ( ex:Female and ex:Human )
+        >>> isinstance(youngWoman,BooleanClass)
+        True
+        >>> isinstance(youngWoman.identifier,BNode)
+        True
         """
         return BooleanClass(operator=OWL_NS.intersectionOf,members=[self,other],graph=self.graph)
             
     def _get_subClassOf(self):
         for anc in self.graph.objects(subject=self.identifier,predicate=RDFS.subClassOf):
-            yield Class(anc,graph=self.graph)
+            yield Class(anc,
+                        graph=self.graph,
+                        skipOWLClassMembership=True)
     def _set_subClassOf(self, other):
         if not other:
             return        
         for sc in other:
             self.graph.add((self.identifier,RDFS.subClassOf,classOrIdentifier(sc)))
-    subClassOf = property(_get_subClassOf, _set_subClassOf)
+            
+    @TermDeletionHelper(RDFS.subClassOf)
+    def _del_subClassOf(self):            
+        pass            
+            
+    subClassOf = property(_get_subClassOf, _set_subClassOf, _del_subClassOf)
 
     def _get_equivalentClass(self):
         for ec in self.graph.objects(subject=self.identifier,predicate=OWL_NS.equivalentClass):
@@ -555,7 +720,12 @@ class Class(AnnotatibleTerms):
             return
         for sc in other:
             self.graph.add((self.identifier,OWL_NS.equivalentClass,classOrIdentifier(sc)))
-    equivalentClass = property(_get_equivalentClass, _set_equivalentClass)
+            
+    @TermDeletionHelper(OWL_NS.equivalentClass)
+    def _del_equivalentClass(self):            
+        pass                        
+            
+    equivalentClass = property(_get_equivalentClass, _set_equivalentClass, _del_equivalentClass)
 
     def _get_disjointWith(self):
         for dc in self.graph.objects(subject=self.identifier,predicate=OWL_NS.disjointWith):
@@ -565,7 +735,12 @@ class Class(AnnotatibleTerms):
             return
         for c in other:
             self.graph.add((self.identifier,OWL_NS.disjointWith,classOrIdentifier(c)))
-    disjointWith = property(_get_disjointWith, _set_disjointWith)
+            
+    @TermDeletionHelper(OWL_NS.disjointWith)
+    def _del_disjointWith(self):            
+        pass            
+                        
+    disjointWith = property(_get_disjointWith, _set_disjointWith, _del_disjointWith)
 
     def _get_complementOf(self):
         comp = list(self.graph.objects(subject=self.identifier,predicate=OWL_NS.complementOf))
@@ -580,18 +755,12 @@ class Class(AnnotatibleTerms):
         if not other:
             return
         self.graph.add((self.identifier,OWL_NS.complementOf,classOrIdentifier(other)))
-    complementOf = property(_get_complementOf, _set_complementOf)
-
-    def _get_seeAlso(self):
-        for ec in self.graph.objects(subject=self.identifier,predicate=RDFS.seeAlso):
-            yield Class(ec,graph=self.graph)
+        
+    @TermDeletionHelper(OWL_NS.complementOf)
+    def _del_complementOf(self):            
+        pass            
                 
-    def _set_seeAlso(self, others):
-        if not others:
-            return
-        for link in others:
-            self.graph.add((self.identifier,RDFS.seeAlso,link))
-    seeAlso = property(_get_seeAlso, _set_seeAlso)
+    complementOf = property(_get_complementOf, _set_complementOf, _del_complementOf)
     
     def isPrimitive(self):
         if (self.identifier,RDF.type,OWL_NS.Restriction) in self.graph:
@@ -627,6 +796,8 @@ class Class(AnnotatibleTerms):
             ec.append(manchesterSyntax(rdfList,self.graph,boolean=p))
         dc = list(self.disjointWith)
         c  = self.complementOf
+        if c:
+            dc.append(c)
         klassKind = ''
         label = list(self.graph.objects(self.identifier,RDFS.label))
         label = label and '('+label[0]+')' or ''
@@ -657,8 +828,6 @@ class Class(AnnotatibleTerms):
             if full:
                 exprs[-1]="\n    "+exprs[-1]
         if dc:
-            if c:
-                dc.append(c)
             exprs.append("DisjointWith %s\n"%'\n                 '.join([
               manchesterSyntax(classOrIdentifier(s),self.graph) for s in dc]))
             if full:
@@ -712,6 +881,9 @@ class OWLRDFListProxy(object):
             if i == classOrIdentifier(item):
                 return 1
         return 0
+    
+    def append(self, item):
+        self._rdfList.append(item)    
 
     def __iadd__(self, other):
         self._rdfList.append(classOrIdentifier(other))
@@ -722,7 +894,31 @@ class EnumeratedClass(Class,OWLRDFListProxy):
     
     OWL Abstract Syntax is used
     
-    axiom ::= 'EnumeratedClass(' classID ['Deprecated'] { annotation } { individualID } ')'    
+    axiom ::= 'EnumeratedClass(' classID ['Deprecated'] { annotation } { individualID } ')'
+    
+
+    >>> exNs = Namespace('http://example.com/')        
+    >>> namespace_manager = NamespaceManager(Graph())
+    >>> namespace_manager.bind('ex', exNs, override=False)
+    >>> namespace_manager.bind('owl', OWL_NS, override=False)
+    >>> g = Graph()    
+    >>> g.namespace_manager = namespace_manager
+    >>> Individual.factoryGraph = g
+    >>> ogbujiBros = EnumeratedClass(exNs.ogbujicBros,
+    ...                              members=[exNs.chime,
+    ...                                       exNs.uche,
+    ...                                       exNs.ejike])
+    >>> ogbujiBros
+    { ex:chime ex:uche ex:ejike }
+    >>> print g.serialize(format='n3')
+    <BLANKLINE>
+    @prefix ex: <http://example.com/>.
+    @prefix owl: <http://www.w3.org/2002/07/owl#>.
+    @prefix rdf: <http://www.w3.org/1999/02/22-rdf-syntax-ns#>.
+    <BLANKLINE>
+     ex:ogbujicBros a owl:Class;
+         owl:oneOf ( ex:chime ex:uche ex:ejike ). 
+        
     """
     _operator = OWL_NS.oneOf
     def isPrimitive(self):
@@ -737,9 +933,52 @@ class EnumeratedClass(Class,OWLRDFListProxy):
         Returns the Manchester Syntax equivalent for this class
         """
         return manchesterSyntax(self._rdfList.uri,self.graph,boolean=self._operator)        
-
+    
+    def serialize(self,graph):
+        clonedList = Collection(graph,BNode())
+        for cl in self._rdfList:
+            clonedList.append(cl)
+            CastClass(cl, self.graph).serialize(graph)
+        
+        graph.add((self.identifier,self._operator,clonedList.uri))
+        for s,p,o in self.graph.triples((self.identifier,None,None)):
+            if p != self._operator:
+                graph.add((s,p,o))
+        self._serialize(graph)
+    
 BooleanPredicates = [OWL_NS.intersectionOf,OWL_NS.unionOf]
 
+class BooleanClassExtentHelper:
+    """
+    >>> testGraph = Graph()
+    >>> Individual.factoryGraph = testGraph
+    >>> EX = Namespace("http://example.com/")
+    >>> namespace_manager = NamespaceManager(Graph())
+    >>> namespace_manager.bind('ex', EX, override=False)
+    >>> testGraph.namespace_manager = namespace_manager
+    >>> fire  = Class(EX.Fire)
+    >>> water = Class(EX.Water) 
+    >>> testClass = BooleanClass(members=[fire,water])
+    >>> testClass2 = BooleanClass(operator=OWL_NS.unionOf,members=[fire,water])
+    >>> for c in BooleanClass.getIntersections():
+    ...     print c
+    ( ex:Fire and ex:Water )
+    >>> for c in BooleanClass.getUnions():
+    ...     print c
+    ( ex:Fire or ex:Water )
+    """    
+    def __init__(self, operator):
+        self.operator = operator
+    def __call__(self, f):
+        def _getExtent():
+            for c in Individual.factoryGraph.subjects(self.operator):
+                yield BooleanClass(c,operator=self.operator)            
+        return _getExtent
+    
+class Callable:
+    def __init__(self, anycallable):
+        self.__call__ = anycallable    
+
 class BooleanClass(Class,OWLRDFListProxy):
     """
     See: http://www.w3.org/TR/owl-ref/#Boolean
@@ -747,6 +986,16 @@ class BooleanClass(Class,OWLRDFListProxy):
     owl:complementOf is an attribute of Class, however
     
     """
+    @BooleanClassExtentHelper(OWL_NS.intersectionOf)
+    @Callable
+    def getIntersections(): pass
+    getIntersections = Callable(getIntersections)    
+
+    @BooleanClassExtentHelper(OWL_NS.unionOf)
+    @Callable
+    def getUnions(): pass
+    getUnions = Callable(getUnions)    
+            
     def __init__(self,identifier=None,operator=OWL_NS.intersectionOf,
                  members=None,graph=None):
         if operator is None:
@@ -765,6 +1014,19 @@ class BooleanClass(Class,OWLRDFListProxy):
         assert not members or not rdfList,"This is a previous boolean class description!"+repr(Collection(self.graph,rdfList[0]).n3())        
         OWLRDFListProxy.__init__(self, rdfList, members)
 
+    def serialize(self,graph):
+        clonedList = Collection(graph,BNode())
+        for cl in self._rdfList:
+            clonedList.append(cl)
+            CastClass(cl, self.graph).serialize(graph)
+        
+        graph.add((self.identifier,self._operator,clonedList.uri))
+        
+        for s,p,o in self.graph.triples((self.identifier,None,None)):
+            if p != self._operator:
+                graph.add((s,p,o))
+        self._serialize(graph)
+
     def isPrimitive(self):
         return False
 
@@ -834,12 +1096,22 @@ class Restriction(Class):
                         OWL_NS.maxCardinality,
                         OWL_NS.minCardinality]
     
-    def __init__(self,onProperty,graph = Graph(),allValuesFrom=None,someValuesFrom=None,value=None,
-                      cardinality=None,maxCardinality=None,minCardinality=None,identifier=None):
+    def __init__(self,
+                 onProperty,
+                 graph = Graph(),
+                 allValuesFrom=None,
+                 someValuesFrom=None,
+                 value=None,
+                 cardinality=None,
+                 maxCardinality=None,
+                 minCardinality=None,
+                 identifier=None):
         super(Restriction, self).__init__(identifier,
                                           graph=graph,
                                           skipOWLClassMembership=True)
-        if (self.identifier,OWL_NS.onProperty,propertyOrIdentifier(onProperty)) not in graph:
+        if (self.identifier,
+            OWL_NS.onProperty,
+            propertyOrIdentifier(onProperty)) not in graph:
             graph.add((self.identifier,OWL_NS.onProperty,propertyOrIdentifier(onProperty)))
         self.onProperty = onProperty
         restrTypes = [
@@ -856,6 +1128,14 @@ class Restriction(Class):
             self.graph.add((self.identifier,oTerm,classOrTerm(val)))   
         if (self.identifier,RDF.type,OWL_NS.Restriction) not in self.graph:
             self.graph.add((self.identifier,RDF.type,OWL_NS.Restriction))
+            self.graph.remove((self.identifier,RDF.type,OWL_NS.Class))
+
+    def serialize(self,graph):
+        Property(self.onProperty,graph=self.graph).serialize(graph)
+        for s,p,o in self.graph.triples((self.identifier,None,None)):
+            graph.add((s,p,o))
+            if p in [OWL_NS.allValuesFrom,OWL_NS.someValuesFrom]:
+                CastClass(o, self.graph).serialize(graph)
 
     def isPrimitive(self):
         return False
@@ -865,7 +1145,7 @@ class Restriction(Class):
         Equivalence of restrictions is determined by equivalence of the property 
         in question and the restriction 'range'
         """
-        assert isinstance(other,Class)
+        assert isinstance(other,Class),repr(other)+repr(type(other))
         if isinstance(other,Restriction):
             return other.onProperty == self.onProperty and \
                    other.restrictionRange == self.restrictionRange
@@ -882,7 +1162,12 @@ class Restriction(Class):
             return
         else:
             self.graph.set(triple)
-    onProperty = property(_get_onProperty, _set_onProperty)
+            
+    @TermDeletionHelper(OWL_NS.onProperty)
+    def _del_onProperty(self):            
+        pass            
+                        
+    onProperty = property(_get_onProperty, _set_onProperty, _del_onProperty)
 
     def _get_allValuesFrom(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.allValuesFrom):
@@ -896,7 +1181,12 @@ class Restriction(Class):
             return
         else:
             self.graph.set(triple)
-    allValuesFrom = property(_get_allValuesFrom, _set_allValuesFrom)
+            
+    @TermDeletionHelper(OWL_NS.allValuesFrom)
+    def _del_allValuesFrom(self):            
+        pass            
+                        
+    allValuesFrom = property(_get_allValuesFrom, _set_allValuesFrom, _del_allValuesFrom)
 
     def _get_someValuesFrom(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.someValuesFrom):
@@ -910,7 +1200,12 @@ class Restriction(Class):
             return
         else:
             self.graph.set(triple)
-    someValuesFrom = property(_get_someValuesFrom, _set_someValuesFrom)
+            
+    @TermDeletionHelper(OWL_NS.someValuesFrom)
+    def _del_someValuesFrom(self):            
+        pass            
+                        
+    someValuesFrom = property(_get_someValuesFrom, _set_someValuesFrom, _del_someValuesFrom)
 
     def _get_hasValue(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.hasValue):
@@ -924,7 +1219,12 @@ class Restriction(Class):
             return
         else:
             self.graph.set(triple)
-    hasValue = property(_get_hasValue, _set_hasValue)
+            
+    @TermDeletionHelper(OWL_NS.hasValue)
+    def _del_hasValue(self):            
+        pass            
+                        
+    hasValue = property(_get_hasValue, _set_hasValue, _del_hasValue)
 
     def _get_cardinality(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.cardinality):
@@ -938,7 +1238,12 @@ class Restriction(Class):
             return
         else:
             self.graph.set(triple)
-    cardinality = property(_get_cardinality, _set_cardinality)
+            
+    @TermDeletionHelper(OWL_NS.cardinality)
+    def _del_cardinality(self):            
+        pass            
+                        
+    cardinality = property(_get_cardinality, _set_cardinality, _del_cardinality)
 
     def _get_maxCardinality(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.maxCardinality):
@@ -952,7 +1257,12 @@ class Restriction(Class):
             return
         else:
             self.graph.set(triple)
-    maxCardinality = property(_get_maxCardinality, _set_maxCardinality)
+            
+    @TermDeletionHelper(OWL_NS.maxCardinality)
+    def _del_maxCardinality(self):            
+        pass            
+                        
+    maxCardinality = property(_get_maxCardinality, _set_maxCardinality, _del_maxCardinality)
 
     def _get_minCardinality(self):
         for i in self.graph.objects(subject=self.identifier,predicate=OWL_NS.minCardinality):
@@ -966,7 +1276,12 @@ class Restriction(Class):
             return
         else:
             self.graph.set(triple)
-    minCardinality = property(_get_minCardinality, _set_minCardinality)
+            
+    @TermDeletionHelper(OWL_NS.minCardinality)
+    def _del_minCardinality(self):            
+        pass            
+                        
+    minCardinality = property(_get_minCardinality, _set_minCardinality, _del_minCardinality)
 
     def restrictionKind(self):
         for p in self.graph.triple_choices((self.identifier,
@@ -1023,10 +1338,32 @@ class Property(AnnotatibleTerms):
         self.range         = range
         self.comment = comment and comment or []
 
+    def serialize(self,graph):
+        for fact in self.graph.triples((self.identifier,None,None)):
+            graph.add(fact)
+        for p in itertools.chain(self.subPropertyOf,
+                                 self.inverseOf):
+            p.serialize(graph)
+        for c in itertools.chain(self.domain,
+                                 self.range):
+            CastClass(c,self.graph).serialize(graph)
+
+    def _get_extent(self,graph=None):
+        for triple in (graph is None and 
+                       self.graph or graph).triples((None,self.identifier,None)):
+            yield triple
+    def _set_extent(self,other):
+        if not other:
+            return
+        for subj,obj in other:
+            self.graph.add((subj,self.identifier,obj))
+            
+    extent = property(_get_extent, _set_extent)            
+
     def __repr__(self):
         rt=[]
         if OWL_NS.ObjectProperty in self.type:
-            rt.append('ObjectProperty( %s annotation(%s)'\
+            rt.append(u'ObjectProperty( %s annotation(%s)'\
                        %(self.qname,first(self.comment) and first(self.comment) or ''))
             if first(self.inverseOf):
                 twoLinkInverse=first(first(self.inverseOf).inverseOf)
@@ -1034,43 +1371,43 @@ class Property(AnnotatibleTerms):
                     inverseRepr=first(self.inverseOf).qname
                 else:
                     inverseRepr=repr(first(self.inverseOf))
-                rt.append("  inverseOf( %s )%s"%(inverseRepr,
-                            OWL_NS.Symmetric in self.type and ' Symmetric' or ''))
+                rt.append(u"  inverseOf( %s )%s"%(inverseRepr,
+                            OWL_NS.Symmetric in self.type and u' Symmetric' or u''))
             for s,p,roleType in self.graph.triples_choices((self.identifier,
                                                             RDF.type,
                                                             [OWL_NS.Functional,
                                                              OWL_NS.InverseFunctionalProperty,
                                                              OWL_NS.Transitive])):
-                rt.append(roleType.split(OWL_NS)[-1])
+                rt.append(unicode(roleType.split(OWL_NS)[-1]))
         else:
             rt.append('DatatypeProperty( %s %s'\
                        %(self.qname,first(self.comment) and first(self.comment) or ''))            
             for s,p,roleType in self.graph.triples((self.identifier,
                                                     RDF.type,
                                                     OWL_NS.Functional)):
-                rt.append('   Functional')
+                rt.append(u'   Functional')
         def canonicalName(term,g):
             normalizedName=classOrIdentifier(term)
             if isinstance(normalizedName,BNode):
                 return term
             elif normalizedName.startswith(_XSD_NS):
-                return term
+                return unicode(term)
             elif first(g.triples_choices((
                       normalizedName,
                       [OWL_NS.unionOf,
                        OWL_NS.intersectionOf],None))):
                 return repr(term)
             else:
-                return str(term.qname)
-        rt.append(' '.join(["   super( %s )"%canonicalName(superP,self.graph) 
+                return unicode(term.qname)
+        rt.append(u' '.join([u"   super( %s )"%canonicalName(superP,self.graph) 
                               for superP in self.subPropertyOf]))                        
-        rt.append(' '.join(["   domain( %s )"% canonicalName(domain,self.graph) 
+        rt.append(u' '.join([u"   domain( %s )"% canonicalName(domain,self.graph) 
                               for domain in self.domain]))
-        rt.append(' '.join(["   range( %s )"%canonicalName(range,self.graph) 
+        rt.append(u' '.join([u"   range( %s )"%canonicalName(range,self.graph) 
                               for range in self.range]))
-        rt='\n'.join([expr for expr in rt if expr])
-        rt+='\n)'
-        return rt
+        rt=u'\n'.join([expr for expr in rt if expr])
+        rt+=u'\n)'
+        return unicode(rt).encode('utf-8')
                     
     def _get_subPropertyOf(self):
         for anc in self.graph.objects(subject=self.identifier,predicate=RDFS.subPropertyOf):
@@ -1080,7 +1417,12 @@ class Property(AnnotatibleTerms):
             return        
         for sP in other:
             self.graph.add((self.identifier,RDFS.subPropertyOf,classOrIdentifier(sP)))
-    subPropertyOf = property(_get_subPropertyOf, _set_subPropertyOf)
+            
+    @TermDeletionHelper(RDFS.subPropertyOf)
+    def _del_subPropertyOf(self):            
+        pass            
+                        
+    subPropertyOf = property(_get_subPropertyOf, _set_subPropertyOf, _del_subPropertyOf)
 
     def _get_inverseOf(self):
         for anc in self.graph.objects(subject=self.identifier,predicate=OWL_NS.inverseOf):
@@ -1089,7 +1431,12 @@ class Property(AnnotatibleTerms):
         if not other:
             return        
         self.graph.add((self.identifier,OWL_NS.inverseOf,classOrIdentifier(other)))
-    inverseOf = property(_get_inverseOf, _set_inverseOf)
+        
+    @TermDeletionHelper(OWL_NS.inverseOf)
+    def _del_inverseOf(self):            
+        pass            
+                
+    inverseOf = property(_get_inverseOf, _set_inverseOf, _del_inverseOf)
 
     def _get_domain(self):
         for dom in self.graph.objects(subject=self.identifier,predicate=RDFS.domain):
@@ -1102,7 +1449,12 @@ class Property(AnnotatibleTerms):
         else:
             for dom in other:
                 self.graph.add((self.identifier,RDFS.domain,classOrIdentifier(dom)))
-    domain = property(_get_domain, _set_domain)
+                
+    @TermDeletionHelper(RDFS.domain)
+    def _del_domain(self):            
+        pass            
+                                
+    domain = property(_get_domain, _set_domain, _del_domain)
 
     def _get_range(self):
         for ran in self.graph.objects(subject=self.identifier,predicate=RDFS.range):
@@ -1115,7 +1467,24 @@ class Property(AnnotatibleTerms):
         else:        
             for range in ranges:
                 self.graph.add((self.identifier,RDFS.range,classOrIdentifier(range)))
-    range = property(_get_range, _set_range)
+                
+    @TermDeletionHelper(RDFS.range)
+    def _del_range(self):            
+        pass            
+                                
+    range = property(_get_range, _set_range, _del_range)
+        
+def CommonNSBindings(graph,additionalNS={}):
+    """
+    Takes a graph and binds the common namespaces (rdf,rdfs, & owl)
+    """
+    namespace_manager = NamespaceManager(graph)
+    namespace_manager.bind('rdfs',RDFS.RDFSNS)
+    namespace_manager.bind('rdf',RDF.RDFNS)
+    namespace_manager.bind('owl',OWL_NS)
+    for prefix,uri in additionalNS.items():
+        namespace_manager.bind(prefix, uri, override=False)
+    graph.namespace_manager = namespace_manager
         
 def test():
     import doctest
